#! /bin/bash
# firmware version: 5.2a
#
# script to update 5.X SLC firmware to 5.2a
#
# this script expects:
#     the / partition and /www partition mounted read-write
#     a single large ramdisk created on /tmp
#     the firmware update file to be unpacked in /tmp/$scrdir
#     the calling script will reboot the SLC when this script is complete
#
# this script:
#     copies all the updated files to /
#     copies all the updated files to /www
#     re-customizes any web pages that have been updated
#     deletes any obsolete files
#     for any existing DTDs that have been updated, converts the associated
#         XML file to conform to the new DTD
#     updates the version in /www/xml/server.xml
#     


#
# Main program
#

fwrev=5.2a
fwrevmajor=5
fwrevminor=2
scrdir=/tmp/slcupdate-db-$fwrev
tmpdir=/tmp/xwstmp
tee="/bin/tee -a /www/fwupdate.txt"

if [ ! -f /www/fwupdate.txt ] ; then
    touch /www/fwupdate.txt
fi

prod=$(/bin/grep \<PRODUCT\> /www/xml/config.xml | /bin/awk -F\> '{ print $2 }' | /bin/awk -F\< ' { print $1 }')

if [ ! -d $scrdir ] ; then
    echo "Firmware update directory $scrdir does not exist." | $tee
    echo "Firmware update failed." | $tee
    exit 1
fi

# make sure everything is flushed before we get started
/bin/sync

/bin/mkdir $tmpdir

cd $scrdir

echo "Installing update for $prod firmware revision $fwrev." | $tee

echo "Checking firmware version..." | $tee
currrev=$(/bin/grep REV /www/xml/server.xml | /bin/awk -F\> '{ print $2 }' | /bin/awk -F\< '{ print $1 }')
if [ $currrev == $fwrev ] ; then
    echo "" | $tee
    echo "*** This $prod is already running firmware version $fwrev." | $tee
    echo "*** The patch will not be installed." | $tee
    echo "" | $tee
    exit 0
fi

currmajor=$(/bin/grep REV /www/xml/server.xml | /bin/awk -F\> '{ print $2 }' | /bin/awk -F\< '{ print $1 }' | /bin/awk -F'[A-Za-z]' '{ print $1 }' | /bin/awk -F. '{ print $1 }' )
currminor=$(/bin/grep REV /www/xml/server.xml | /bin/awk -F\> '{ print $2 }' | /bin/awk -F\< '{ print $1 }' | /bin/awk -F'[A-Za-z]' '{ print $1 }' | /bin/awk -F. '{ print $2 }' )

if [ $currmajor -lt 5 ] ; then
    echo "" | $tee
    echo "*** This patch may only be installed on" | $tee
    echo "*** a $prod running firmware version 5.0 or later." | $tee
    echo "*** Please upgrade to one of these firmware versions," | $tee
    echo "*** then reapply this patch." | $tee
    echo "" | $tee
    exit 0
fi

installpatch=1
if [ $currmajor -gt $fwrevmajor ] ; then
    installpatch=0
elif [ $currmajor -eq $fwrevmajor -a $currminor -gt $fwrevminor ] ; then
    installpatch=0
fi
if [ $installpatch -eq 0 ] ; then
    echo "" | $tee
    echo "*** The currently installed firmware version" | $tee
    echo "*** is greater than the patch version." | $tee
    echo "*** The patch will not be installed." | $tee
    echo "" | $tee
    exit 0
fi

echo "Checking filesystem version..." | $tee
if [ -f /sbin/sfdisk ] ; then
    /sbin/sfdisk -l -uS /dev/hda > /tmp/sectors
else
    ./sbin/sfdisk -l -uS /dev/hda > /tmp/sectors
fi
start=$(/bin/grep hda3 /tmp/sectors | /bin/awk ' { print $2 }')
if [ -f uImage ] ; then
    if [ $start -eq 242688 ] ; then
        echo "" | $tee
        echo "*** This patch may not be installed on a $prod " | $tee
        echo "*** that originally ran firmware version 1.0." | $tee
        echo "*** Please contact Lantronix Tech Support." | $tee
        echo "" | $tee
        exit 0
    fi
fi

echo -n "Copying new and changed files..." | $tee

# tar files to the 1st partition
for d in bin boot etc lib root sbin usr var x15 ; do
    if [ -d $d ] ; then
        cd $d
        tar -cf - * | (cd /$d ; tar xf -)
        echo -n "." | $tee
        cd ..
    fi
done

# if there is a u-boot.bin file, touch .updateuboot so that the boot loader
# version will be checked at reboot
if [ -f bin/u-boot.bin ] ; then
    touch /www/.updateuboot
fi

# lots of updates, let's flush again
/bin/sync

# tar files to the 2nd (read-write) partition
if [ -d www ] ; then
    cd www

    if [ -d help -a $prod == "SLC" ] ; then
        tar -cf - help | (cd /www ; tar xf -)
        rm -rf help help-slb
    fi
    if [ -d help-slb -a $prod == "SLB" ] ; then
        if [ ! -d /www/help-slb ] ; then
            /bin/mv /www/help /www/help-slb
            tar -cf - help-slb | (cd /www ; tar xf -)
            /bin/mv /www/help-slb /www/help 
        else
            tar -cf - help-slb | (cd /www ; tar xf -)
        fi
        rm -rf help help-slb
    fi

    # copy all the new/changed files
    tar -cf - * | (cd /www ; tar xf -)

    echo "." | $tee

    # customize web files
    # read values from config.xml, to be used for customizations
    company=$(/bin/grep COMPANY /www/xml/config.xml | /bin/awk -F\> '{ print $2 }' | /bin/awk -F\< ' { print $1 }')
    product=$(/bin/grep \<PRODUCT\> /www/xml/config.xml | /bin/awk -F\> '{ print $2 }' | /bin/awk -F\< ' { print $1 }')
    productlong=$(/bin/grep PRODUCTLONG /www/xml/config.xml | /bin/awk -F\> '{ print $2 }' | /bin/awk -F\< ' { print $1 }')
    model=$(/bin/grep MODEL /www/xml/config.xml | /bin/awk -F\> '{ print $2 }' | /bin/awk -F\< ' { print $1 }')
    website=$(/bin/grep WEBSITE /www/xml/config.xml | /bin/awk -F\> '{ print $2 }' | /bin/awk -F\< ' { print $1 }')
    echo -e "Re-customizing with $company, $product, $productlong,\n                    $model, $website" | $tee

    # determine number of ports
    if [ $model == "SLC8" -o $model == "SLB884" ] ; then
        numports=8
    elif [ $model == "SLC16" -o $model == "SLB1684" ] ; then
        numports=16
    elif [ $model == "SLC32" ] ; then
        numports=32
    else
        numports=48
    fi

    # customize files under /www
    for f in *.htm ; do
        /bin/sed -i "s/Lantronix SLC48/$company $model/g
             s/SecureLinx Console Manager/$productlong/g
             s/SLC48/$model/g
             s/www.lantronix.com/$website/g
             s/SLC /$product /g" /www/$f 
    done
    # web ssh/telnet pages
    if [ -f network/webssh.jnlp -o network/webtelnet.jnlp ] ; then
        cd network
        for f in *.jnlp ; do
            /bin/sed -i "s/Lantronix SLC48/$company $model/g" /www/network/$f
        done
        cd ..
    fi

    # Update pages that are dependent on number of device ports
    if [ -f banner.htm ] ; then
        # remove extra ports from banner.htm -
        # update the file that was just customized
        /bin/rm -f banner.htm $tmpdir/banner.htm
        /bin/cp /www/banner.htm .
        /bin/awk "{ if ( NF >= 8 && \$6 == \"Device\" && \$7 == \"Port\" )
                 { if ( \$8 <= $numports )
                     print \$0 }
               else
                 print \$0 }" banner.htm > $tmpdir/banner.htm
        /bin/rm -f /www/banner.htm
        /bin/cp $tmpdir/banner.htm /www
    fi

    # remove buttons from devports.htm - update the file that was just updated
    if [ -f devports.htm ] ; then
        /bin/rm -f devports.htm $tmpdir/devports.htm
        /bin/cp /www/devports.htm .
        /bin/awk "{ if ( NF >= 3 && \$3 == \"name=btn16\" )
                 { if ( $numports >= 32 )
                     print \$0 }
               else if ( NF >= 3 && \$3 == \"name=btn32\" )
                 { if ( $numports >= 32 )
                     print \$0 }
               else if ( NF >= 3 && \$3 == \"name=btn48\" )
                 { if ( $numports >= 48 )
                     print \$0 }
               else if ( NF >= 2 && \$2 == \"SLC8\" )
                 { if ( $numports == 8 )
                     print \$0 }
               else
                 print \$0 }" devports.htm > $tmpdir/devports.htm
        /bin/rm -f /www/devports.htm
        /bin/cp $tmpdir/devports.htm /www
    fi

    # populate timezones in datetime.htm - update the file that was just updated
    if [ -f datetime.htm -o -f qsetup.htm ] ; then
        dodatetime=0
        doqsetup=0
        if [ -f datetime.htm ] ; then
            dodatetime=1
        fi
        if [ -f qsetup.htm ] ; then
            doqsetup=1
        fi
        /bin/rm -f datetime.htm $tmpdir/datetime.htm
        /bin/cp /www/datetime.htm .
        /bin/rm -f qsetup.htm $tmpdir/qsetup.htm
        /bin/cp /www/qsetup.htm .

        echo "Updating timezones..." | $tee
        # get list of timezone files
        cd /usr/share/zoneinfo
        /bin/rm -f $tmpdir/zonefiles $tmpdir/zonefiles2
        /bin/find . -type f -print | egrep -v 'right|posix|\.tab' | sort > $tmpdir/zonefiles
        # remove leading './'
        /bin/sed 's/\.\///g' $tmpdir/zonefiles > $tmpdir/zonefiles2
        cd $scrdir/www
        /bin/xws_inserttz >> /www/fwupdate.txt 2>&1
        retval=$?
        if [ $retval -eq 0 ]; then
            if [ $dodatetime -eq 1 ] ; then
                /bin/rm -f /www/datetime.htm
                /bin/cp $tmpdir/datetime.htm /www
            fi
            if [ $doqsetup -eq 1 ] ; then
                /bin/rm -f /www/qsetup.htm
                /bin/cp $tmpdir/qsetup.htm /www
            fi
        elif [ $retval -eq 1 ]; then
            echo "Cannot open original datetime.htm or qsetup.htm" | $tee
        elif [ $retval -eq 2 ]; then
            echo "Cannot open new datetime.htm or qsetup.htm" | $tee
        elif [ $retval -eq 3 ]; then
            echo "Cannot open temporary timezone list file" | $tee
        fi
    fi

    cd $scrdir

    # trim xml-defaults devports.xml if number of ports is less than 48
    if [ -f www/xml-defaults/devports.xml -a $numports -ne 48 ]; then
        mkdir /tmp/xml
        cp www/xml-defaults/devports.xml /tmp/xml
        cd /tmp
        /bin/xws_trimdevports $numports >> /www/fwupdate.txt 2>&1
        retval=$?
        if [ $retval -eq 0 ]; then
            /bin/rm -f /www/xml-defaults/devports.xml
            /bin/cp $tmpdir/devports.xml /www/xml-defaults
        elif [ $retval -eq 1 ]; then
            echo "Wrong arguments passed to xws_trimdevports" | $tee
        elif [ $retval -eq 2 ]; then
            echo "Cannot open original xml/devports.xml" | $tee
        elif [ $retval -eq 3 ]; then
            echo "Cannot open new devports.xml" | $tee
        fi
    fi

    cd $scrdir

fi

# delete files
echo -n "Deleting obsolete files..." | $tee
if [ -f delete_files ] ; then
    for f in `cat delete_files` ; do
        rm -f -r $f
        echo -n "." | $tee
    done
fi
echo "" | $tee

# another flush...
/bin/sync


# update kernel
if [ -f uImage ] ; then
    echo "Updating kernel..." | $tee
    /bin/dd if=uImage of=/dev/hda3 >> /www/fwupdate.txt 2>&1
fi

# convert all XML files to 5.2a format

cd $scrdir
for i in config devports kerberos ldap localusers network nis radius remoteusers services tacacs ; do
    case "$i" in
        config)         grepstr="SLB884"
                        xmlstr="Configuration"
                        ;;
        devports)       grepstr="WEBCOLS"
                        xmlstr="Device Ports"
                        ;;
        kerberos)       grepstr="OUTLETS"
                        xmlstr="Kerberos"
                        ;;
        ldap)           grepstr="OUTLETS"
                        xmlstr="LDAP"
                        ;;
        localusers)     grepstr="OUTLETS"
                        xmlstr="Local Users"
                        ;;
        network)        grepstr="ALTERNATE"
                        xmlstr="Network"
                        ;;
        nis)            grepstr="OUTLETS"
                        xmlstr="NIS"
                        ;;
        radius)         grepstr="OUTLETS"
                        xmlstr="RADIUS"
                        ;;
        remoteusers)    grepstr="OUTLETS"
                        xmlstr="Remote Users"
                        ;;
        services)       grepstr="TRAPCOMMUNITY"
                        xmlstr="Services"
                        ;;
        tacacs)         grepstr="OUTLETS"
                        xmlstr="TACACS"
                        ;;
        *)              ;;
    esac
    if /bin/grep -q $grepstr /www/xml/$i.xml ; then
        echo "$xmlstr XML already converted, skipping..." | $tee
    else
        # run conversion program to convert XML to the new DTD
        echo "Converting $xmlstr XML..." | $tee
        ./convert_$i /www/xml/$i.xml
        retval=$?
        if [ $retval -eq 0 ]; then
            /bin/rm -f /www/xml/$i.xml
            /bin/cp /tmp/$i.xml /www/xml
        elif [ $retval -eq 1 ]; then
            echo "Cannot open original /www/xml/$i.xml" | $tee
        elif [ $retval -eq 2 ]; then
            echo "Cannot open new /tmp/$i.xml" | $tee
        fi
    fi
done


# update firmware revision in server.xml
/bin/sed -i "s/<REV>[\.a-zA-Z0-9]*<\/REV>/<REV>$fwrev<\/REV>/g" /www/xml/server.xml >> /www/fwupdate.txt 2>&1
/bin/sed -i "s/<REV>[\.a-zA-Z0-9]*<\/REV>/<REV>$fwrev<\/REV>/g" /www/xml-defaults/server.xml >> /www/fwupdate.txt 2>&1


# touch timestamp file
/bin/touch /www/.update-timestamp

# flush updates
/bin/sync

/bin/echo "Installation of $prod firmware revision $fwrev complete." | $tee

/bin/echo "0Firmware Update..." > /dev/frontpanel0
/bin/echo "1Update Complete." > /dev/frontpanel0

/bin/echo "Rebooting the $prod..." | $tee
/bin/echo "0Update Complete." > /dev/frontpanel0
/bin/echo "1Rebooting the $prod..." > /dev/frontpanel0

/bin/umount -a
/sbin/shutdown -r -q now

exit 0
